#!/bin/bash
#

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e10|e14|i1> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=vecgeom
origpkgver=v0_3_rc
pkgver=${origpkgver}a
ssibuildshims_version=v0_15_08
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarfile=vecgeom-${pkgdotver}.tar

get_this_dir

get_ssibuildshims

source define_basics

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${extraqual} in
debug) extra_command="-DCMAKE_BUILD_TYPE=Debug";;
prof)  extra_command="-DCMAKE_BUILD_TYPE=MinSizeRel";;
* )
   echo "ERROR: please specify debug, or prof"
   usage
   exit 1
;;
esac

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

# We need cmake
source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake || ssi_die "ERROR: failed to setup camke"

if [[ "${basequal}" == e1[024] ]]
then
  cxxflg="-std=c++14"
  extra_command="${extra_command} -DCMAKE_CXX_FLAGS=${cxxflg} -DCMAKE_CXX_STANDARD=14"
else
  ssi_die "Qualifier $basequal not recognized."
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x
cd ${sourcedir}
tar xf ${tardir}/${pkgtarfile}

# copy test files 
cp -p ${patchdir}/cms2015.root ${sourcedir}/${package}-${pkgdotver}/test/cmstestdata/
cp -p ${patchdir}/ExN03.root ${blddir}/

# make sure we aren't using -march=native
patch -b -p0 < ${patchdir}/vecgeom-${pkgdotver}.patch

cd ${blddir}

env CC=gcc CXX=g++ FC=gfortran cmake \
          -DCMAKE_INSTALL_PREFIX=${pkgdir} \
	  -DBACKEND=Scalar -DGEANT4=OFF -DUSOLIDS=ON -DUSOLIDS_VECGEOM=OFF \
	  ${extra_command} \
	  ${sourcedir}/${package}-${pkgdotver}
(( $? == 0 )) || ssi_die "ERROR: cmake failed."
make -j ${ncore} || ssi_die "ERROR: make failed."
make test || ssi_die "ERROR: make test failed."
make install  || ssi_die "ERROR: make install failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "vecgeom is installed at ${VECGEOM_DIR}"

# fix paths for cmake config
cmakeConfig="${VECGEOM_LIB}/cmake/VecCore/VecCoreConfig.cmake ${VECGEOM_LIB}/CMake/VecGeom/VecGeomConfig.cmake ${VECGEOM_LIB}/CMake/USolids/USolidsConfig.cmake"
for file in ${cmakeConfig}
do
  mv ${file} ${file}.bak
  cat ${file}.bak | sed -e "s%${VECGEOM_FQ_DIR}%\$ENV{VECGEOM_FQ_DIR}%g" > ${file}
done


# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
